/*!
 * loadImage.js
 * author: Brent M. Dingle
 * last modified: 2015
 * Example script for simple image loading.
 *
 * Note currently resizing the browser window does NOT change the canvas size
 * So... probably want a boundary edge displayed on the canvas (via html style)
 */
 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theProgram (singleton) Object                            theProgram Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theProgram = 
{
    // ------------------------------------------------------------------------
    // Pseudo-constants
    // ------------------------------------------------------------------------
    IMAGE_CANVAS_ID: "mainImageCanvas",     // canvas id used in html
    
    // ------------------------------------------------------------------------
    // Variables
    // ------------------------------------------------------------------------
    width:          400,           // canvas width... likely will be reset
    height:         400,           // canvas height... likely will be reset
    imageObj:       null,
    
    xOffset:        5,
    yOffset:        5,
    
    // ------------------------------------------------------------------------
    // Functions
    // ------------------------------------------------------------------------
    //                                                          Main
    // ------------------------------------------------------------------------
    Main: function() 
    {
        var canvas = document.getElementById(this.IMAGE_CANVAS_ID);
		canvas.addEventListener('click', theProgram.onClick, false);
		canvas.addEventListener('touch', theProgram.onClick, false);
        
        this.imageObj = new Image();
        this.imageObj.onload = function() 
                                        {
                                            theProgram.drawOrigImage();
                                        };
        this.imageObj.src = 'bobcatCactus.png';
    },
    
    // ------------------------------------------------------------------------
    // EventHandler                                             onClick
    // ------------------------------------------------------------------------
    onClick: function(e)
    {
        theProgram.drawOrigImage();
        theProgram.drawAlteredImage();
    },
    
    // ------------------------------------------------------------------------
    // drawOrigImage
    // ------------------------------------------------------------------------
    drawOrigImage: function()
    {
        var canvas = document.getElementById(this.IMAGE_CANVAS_ID);
        var ctx = canvas.getContext('2d');
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        
        ctx.drawImage(this.imageObj, this.xOffset, this.yOffset);

        ctx.font="16px Georgia";
        ctx.fillText("Click/tap to see the", this.imageObj.width+20, 80);
        ctx.fillText("altered image", this.imageObj.width+20, 100);
    },
    

    // ------------------------------------------------------------------------
    // drawAlteredImage  -- must call drawOrigImage before this
    // ------------------------------------------------------------------------
    drawAlteredImage: function()
    {
        var canvas = document.getElementById(this.IMAGE_CANVAS_ID);
        var ctx = canvas.getContext('2d');

        // Get the image data from the canvas context
        var imageData = ctx.getImageData(this.xOffset, this.yOffset, 
                                         this.imageObj.width, this.imageObj.height);
        var data = imageData.data;

        // This gives us the image data in RGBA format - one byte for each
        // So to iterate over all pixels we would do:
        //for(var i = 0, n = data.length; i < n; i += 4) 
        // {
            // var red = data[i];
            // var green = data[i + 1];
            // var blue = data[i + 2];
            // var alpha = data[i + 3];
        // }
        
        // We must 'create' a new image
        var SecondImageData = ctx.createImageData(this.imageObj.width, this.imageObj.height);

        //var avgColor = 0;
        for (var pixelIndex=0; pixelIndex < SecondImageData.data.length; pixelIndex += 4)
        {
            // Challenge: Apply luminance algorithm of: 0.3*red + 0.59*green + 0.11*blue
            // Hint use: avgColor = Math.round( 0.3*data[pixelIndex] + 0.59*data[pixelIndex +1] + 0.11*data[pixelIndex +2]);
            SecondImageData.data[pixelIndex    ] = data[pixelIndex] * 3;     // More red
            SecondImageData.data[pixelIndex + 1] = data[pixelIndex +1] / 2;  // Less green
            SecondImageData.data[pixelIndex + 2] = data[pixelIndex +2] / 2;  // Less blue
            SecondImageData.data[pixelIndex + 3] = data[pixelIndex +3];      // alpha unchanged
        }
        
        // Want to draw the altered image to the right of original
        // Offset gives spacing between
        var SecondImageX = 2*this.xOffset + this.imageObj.width;
        var SecondImageY = this.yOffset;
        ctx.putImageData(SecondImageData, SecondImageX, SecondImageY);
        
    },
    
};  // end theProgram variable

// ----------------------------------------------------------------------------
//                                                          window.ONLOAD
// ----------------------------------------------------------------------------
window.onload = function()
{
    // Initialize and Start the game
    theProgram.Main();
    
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end of file
